package interceptor;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.AnnotatedType;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.enterprise.inject.spi.InjectionTarget;

import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;

public class InterceptorTest {

	public static void main(String[] args) throws Exception {

		// Bootstrap the CDI environment
		
		WeldContainer weld = new Weld().initialize();
		BeanManager bm = weld.instance().select(BeanManager.class).get();

		// Here we create an instance of InterceptedBean using an
		// InjectionTarget

		AnnotatedType<InterceptedBean> type = bm
				.createAnnotatedType(InterceptedBean.class);
		InjectionTarget<InterceptedBean> injectionTarget = bm
				.createInjectionTarget(type);
		CreationalContext<InterceptedBean> ctx = bm
				.createCreationalContext(null);
		InterceptedBean interceptedBean = injectionTarget.produce(ctx);
		injectionTarget.inject(interceptedBean, ctx);
		injectionTarget.postConstruct(interceptedBean);

		// The bug manifests in the below method execution - instead of invoking
		// the interceptor, the raw method is invoked directly and produces the
		// command line output: alas, no interceptor!
		interceptedBean.interceptedMethod();

		
		
		// Here we create an instance of InterceptedBean using the BeanManager
		// to show that the interceptor works in the "usual" case

		Bean<?> interceptedBeanBean = bm.resolve(bm
				.getBeans(InterceptedBean.class));
		InterceptedBean bean2 = (InterceptedBean) bm.getReference(
				interceptedBeanBean, InterceptedBean.class, bm
						.createCreationalContext(null));
		
		// Invoking the below correctly invokes the interceptor and produces
		// on the command line: invoked interceptor!
		bean2.interceptedMethod();

	}

}